#ifndef _RICK_H                // Prevent multiple definitions if this 
#define _RICK_H                // file is included in more than one place
#define WIN32_LEAN_AND_MEAN

#include "sprite.h"
#include "Zombie.h"



class Rick : Sprite
{
	private:
		static const int NCOLS = 4;
		static const int HEIGHT = TILE_HEIGHT;
		static const int WIDTH = TILE_WIDTH;
		unsigned int curMoveTime;
		unsigned int lastMoveTime;
		unsigned int curAttackTime;
		unsigned int lastAttackTime;
		int genPathCalls;
		TextureManager texture;

		void generatePath(int, int,
			              int[GAME_TILE_WIDTH][GAME_TILE_HEIGHT],
			              int[GAME_TILE_WIDTH][GAME_TILE_HEIGHT]);
		bool isGood(int, int,
			        int[GAME_TILE_WIDTH][GAME_TILE_HEIGHT],
					int[GAME_TILE_WIDTH][GAME_TILE_HEIGHT],
					int);
		void attack( short, int[GAME_TILE_WIDTH][GAME_TILE_HEIGHT], Zombie[16] );

	public:
		bool initialize(int[GAME_TILE_WIDTH][GAME_TILE_HEIGHT],
			            int[GAME_TILE_WIDTH][GAME_TILE_HEIGHT],
						Graphics*);
		void update(int[GAME_TILE_WIDTH][GAME_TILE_HEIGHT],
			        int[GAME_TILE_WIDTH][GAME_TILE_HEIGHT],
					Zombie[16],
					Input*, float);
		void draw();
		void onLostDevice();
		void onResetDevice();
		int  getX()        {return spriteData.x;}
		int  getY()        {return spriteData.y;}
		int  health;
	 

};

void Rick::generatePath(int x, int y, int map[GAME_TILE_WIDTH][GAME_TILE_HEIGHT], 
	                    int path[GAME_TILE_WIDTH][GAME_TILE_HEIGHT])
{
	genPathCalls++;
	if(path[x][y] < GAME_TILE_WIDTH + GAME_TILE_HEIGHT - 1)
	{
		//Rick generates paths to himself.
		//This makes finding paths more quick for multiple zombies.
		if( isGood(x-1,y, map, path, path[x][y] )) 
		{
			path[x-1][y] = path[x][y]+1;
			generatePath( x-1, y, map, path);
		}
		if( isGood(x+1,y, map, path, path[x][y] )) 
		{
			path[x+1][y] = path[x][y]+1;
			generatePath( x+1, y, map, path);
		}
		if( isGood(x,y-1, map, path, path[x][y] )) 
		{
			path[x][y-1] = path[x][y]+1;
			generatePath( x, y-1, map, path);
		}
		if( isGood(x,y+1, map, path, path[x][y] )) 
		{
			path[x][y+1] = path[x][y]+1;
			generatePath( x, y+1, map, path);
		}
	}
}

bool Rick::isGood(int x, int y,
	              int map[GAME_TILE_WIDTH][GAME_TILE_HEIGHT],
				  int path[GAME_TILE_WIDTH][GAME_TILE_HEIGHT],
				  int count)
{
	if(x>=0 && x < GAME_TILE_WIDTH &&
	   y>=0 && y < GAME_TILE_HEIGHT && 
	   map[x][y] == -3 &&
	   (path[x][y] > count || path[x][y] == -1))
	{
		return true;
	}
	return false;
}

void Rick::attack(short direc, int map[GAME_TILE_WIDTH][GAME_TILE_HEIGHT], Zombie zombies[16])
{
	if(direc == DIREC_UP)
	{
		if(map[spriteData.x][spriteData.y-1] >= 0)
		{
			zombies[map[spriteData.x][spriteData.y-1]].health -= 1;
			
		}
	}
	else if(direc == DIREC_DOWN)
	{
		if(map[spriteData.x][spriteData.y+1] >= 0)
		{
			zombies[map[spriteData.x][spriteData.y+1]].health -= 1;
		
		}
	}
	else if(direc == DIREC_LEFT)
	{
		if(map[spriteData.x-1][spriteData.y] >= 0)
		{
			zombies[map[spriteData.x-1][spriteData.y]].health -= 1;
			
		}
	}
	else if(direc == DIREC_RIGHT)
	{
		if(map[spriteData.x+1][spriteData.y] >= 0)
		{
			zombies[map[spriteData.x+1][spriteData.y]].health -= 1;
			
		}
	}
}

bool Rick::initialize(int map[GAME_TILE_WIDTH][GAME_TILE_HEIGHT],
	                  int path[GAME_TILE_WIDTH][GAME_TILE_HEIGHT],
					  Graphics *g)
{
	genPathCalls = 0;
	health = 5;
	lastAttackTime = lastMoveTime = GetTickCount();
	if (!texture.initialize(g,RICK_IMAGE))
        throw(GameError(gameErrorNS::FATAL_ERROR, "Error initializing Rick's texture"));
	Sprite::initialize(g, WIDTH, HEIGHT, NCOLS, &texture);
	Sprite::setCurrentFrame(12);     // starting frame
    Sprite::setFrameDelay(0.1f);
	if( map[2][3] == -3)
	{
		spriteData.tiled = true;
		spriteData.x = 2;
		spriteData.y = 3;
		map[2][3] = -1;
		path[2][3] = 0;
		generatePath(2, 3, map, path);
	}
	else
		throw(GameError(gameErrorNS::FATAL_ERROR, "Error placing Rick."));
}

void Rick::update(int map[GAME_TILE_WIDTH][GAME_TILE_HEIGHT],
	              int path[GAME_TILE_WIDTH][GAME_TILE_HEIGHT],
				  Zombie zombies[16],
		          Input* input, float frameTime)
{
	if(getDirec() == DIREC_RIGHT)
	{
		spriteData.flipHorizontal = true;
	}
	else
	{
		spriteData.flipHorizontal = false;
	}
	genPathCalls = 0;
	bool needNewPath = false;
	curAttackTime = curMoveTime = GetTickCount();
	if(curMoveTime - lastMoveTime >= 600 )
	{
		if (getDirec()==DIREC_RIGHT)
		{
			Sprite::setFrames(20, 20);
			Sprite::setCurrentFrame(20);
		}
		else if (getDirec()==DIREC_LEFT)
		{
			Sprite::setFrames(24, 24);
			Sprite::setCurrentFrame(24);
		}
		else if (getDirec()==DIREC_UP)
		{
			Sprite::setFrames(16, 16);
			Sprite::setCurrentFrame(16);
		}
		else
		{
			Sprite::setFrames(12, 12);
			Sprite::setCurrentFrame(12);
		}

		if(input->isKeyDown(RIGHT_KEY))            // if move right
		{
				setDirec(DIREC_RIGHT);
				Sprite::setFrames(20, 23);
				Sprite::setCurrentFrame(20);     // starting frame
				if(spriteData.x+1 >= GAME_TILE_WIDTH)//if past screen end
				{
					if(map[0][spriteData.y] == -3)//if next tile is open
					{
						needNewPath = true;
						lastMoveTime = curMoveTime;
						map[spriteData.x][spriteData.y] = -3; //free current spot
						spriteData.x = 0;                    
						map[spriteData.x][spriteData.y] = -1;
					}
				}
				else
				{
					if(map[spriteData.x+1][spriteData.y] == -3) //if next tile is open
					{
						needNewPath = true;
						lastMoveTime = curMoveTime;
						map[spriteData.x][spriteData.y] = -3; //free current spot
						spriteData.x++;
						map[spriteData.x][spriteData.y] = -1;
					}
				}
		}
		if(input->isKeyDown(LEFT_KEY))             // if move left
		{
				setDirec(DIREC_LEFT);
				Sprite::setFrames(24, 27);
				Sprite::setCurrentFrame(24);     // starting frame
				if(spriteData.x-1 < 0)//if past screen end
				{
					if(map[GAME_TILE_WIDTH-1][spriteData.y] == -3)//if next tile is open
					{
						needNewPath = true;
						lastMoveTime = curMoveTime;
						map[spriteData.x][spriteData.y] = -3; //free current spot
						spriteData.x = GAME_TILE_WIDTH-1;                    
						map[spriteData.x][spriteData.y] = -1;
					}
				}
				else
				{
					if(map[spriteData.x-1][spriteData.y] == -3)
					{
						needNewPath = true;
						lastMoveTime = curMoveTime;
						map[spriteData.x][spriteData.y] = -3; //free current spot
						spriteData.x--;
						map[spriteData.x][spriteData.y] = -1;
					}
				}
		}
		if(input->isKeyDown(UP_KEY))               // if move up
		{
				setDirec(DIREC_UP);
				Sprite::setFrames(16, 19);
				Sprite::setCurrentFrame(16);     // starting frame
	
				if(spriteData.y <= 0)//if past screen end
				{
					if(map[spriteData.x][GAME_TILE_HEIGHT-1] == -3)//if next tile is open
					{
						needNewPath = true;
						lastMoveTime = curMoveTime;
						map[spriteData.x][spriteData.y] = -3; //free current spot
						spriteData.y = GAME_TILE_HEIGHT-1;                    
						map[spriteData.x][spriteData.y] = -1;
					}
				}
				else
				{
					if(map[spriteData.x][spriteData.y-1] == -3) //if next tile is open
					{
						needNewPath = true;
						lastMoveTime = curMoveTime;
						map[spriteData.x][spriteData.y] = -3; //free current spot
						spriteData.y--;
						map[spriteData.x][spriteData.y] = -1;
					}
				}
		}
		if(input->isKeyDown(DOWN_KEY))             // if move down
		{
				setDirec(DIREC_DOWN);
				Sprite::setFrames(12, 15);
				Sprite::setCurrentFrame(12);     // starting frame
				if(spriteData.y+1 >= GAME_TILE_HEIGHT)//if past screen end
				{
					if(map[spriteData.x][0] == -3)//if next tile is open
					{
						needNewPath = true;
						lastMoveTime = curMoveTime;
						map[spriteData.x][spriteData.y] = -3; //free current spot
						spriteData.y = 0;                    
						map[spriteData.x][spriteData.y] = -1;
					}
				}
				else
				{
					if(map[spriteData.x][spriteData.y+1] == -3) //if next tile is open
					{
						needNewPath = true;
						lastMoveTime = curMoveTime;
						map[spriteData.x][spriteData.y] = -3; //free current spot
						spriteData.y++;
						map[spriteData.x][spriteData.y] = -1;
					}
				}
		}
	}

	if(input->isKeyDown(SPACE_KEY) && curAttackTime - lastAttackTime >= 200)
	{
		lastAttackTime = curAttackTime;
		attack( getDirec(), map, zombies );
	}

	//if( needNewPath )
	//{
		for(int i=0; i<GAME_TILE_WIDTH; i++)
		{
			for(int j=0; j<GAME_TILE_HEIGHT; j++)
			{
				path[i][j] = -1;
			}
		}
		path[spriteData.x][spriteData.y] = 0;
		generatePath(spriteData.x, spriteData.y, map, path);
		genPathCalls;
	//}
    Sprite::update(frameTime);
}

void Rick::draw()
{
	Sprite::draw();
}

void Rick::onLostDevice()
{
	texture.onLostDevice();
}
void Rick::onResetDevice()
{
	texture.onResetDevice();
}


#endif