#ifndef _ZOMBIE_H                // Prevent multiple definitions if this 
#define _ZOMBIE_H                // file is included in more than one place
#define WIN32_LEAN_AND_MEAN

#include "sprite.h"

class Zombie : Sprite
{
	private:
		static const int NCOLS = 3;
		static const int HEIGHT = TILE_HEIGHT;
		static const int WIDTH = TILE_WIDTH;
		unsigned int curMoveTime;
		unsigned int lastMoveTime;
		unsigned int lastAttackTime;
		unsigned int curAttackTime;
		TextureManager texture;

		int getValue(int, int, int[GAME_TILE_WIDTH][GAME_TILE_HEIGHT]);

	public:
		Zombie();
		Zombie(int, int, int[GAME_TILE_WIDTH][GAME_TILE_HEIGHT], int, Graphics*);
		bool initialize(int, int, int[GAME_TILE_WIDTH][GAME_TILE_HEIGHT], int, Graphics*);
		void update(int[GAME_TILE_WIDTH][GAME_TILE_HEIGHT], int,
			        int[GAME_TILE_WIDTH][GAME_TILE_HEIGHT], Input*, float, int&);
		void draw();
		void onLostDevice();
		void onResetDevice();
		int getX();
		int getY();
		void setX(int);
		void setY(int);
		int health;
		bool alive;
	

};

Zombie::Zombie()
{
	Sprite();
}

Zombie::Zombie(int x, int y, int map[GAME_TILE_WIDTH][GAME_TILE_HEIGHT], int index, Graphics *g)
{
	Sprite();
	initialize(x, y, map, index, g);
}

int Zombie::getValue(int x, int y, int path[GAME_TILE_WIDTH][GAME_TILE_HEIGHT])
{
	if(x>=0 && x < GAME_TILE_WIDTH && y>=0 && y < GAME_TILE_HEIGHT )
		return path[x][y];
	return -1;
}

bool Zombie::initialize(int x, int y, int map[GAME_TILE_WIDTH][GAME_TILE_HEIGHT], int index, Graphics *g)
{
	lastMoveTime = GetTickCount();
	if (!texture.initialize(g,ZOMBIE_IMAGE))
        throw(GameError(gameErrorNS::FATAL_ERROR, "Error initializing Zombies's texture"));
	Sprite::initialize(g, WIDTH, HEIGHT, NCOLS, &texture);
	if( map[x][y] == -3)
	{
		spriteData.tiled = true;
		spriteData.x = x;
		spriteData.y = y;
		map[x][y] = index;
	}
	health = 1;
	alive = true;
}

void Zombie::update(int map[GAME_TILE_WIDTH][GAME_TILE_HEIGHT], int index,
	                int path[GAME_TILE_WIDTH][GAME_TILE_HEIGHT],
					Input* input, float frameTime, int &attacks)
{
	if(alive)
	{
		curAttackTime = curMoveTime = GetTickCount();
		if( curMoveTime - lastMoveTime > 800 )
		{
			int valLeft = getValue(spriteData.x-1,spriteData.y,path);
			int valRight = getValue(spriteData.x+1,spriteData.y,path);
			int valUp = getValue(spriteData.x,spriteData.y-1,path);
			int valDown = getValue(spriteData.x,spriteData.y+1,path);
			if( curAttackTime - lastAttackTime > 500 )
			{
				lastAttackTime = curAttackTime;
				if( valLeft == 0 )
				{
					attacks++;
				}
				else if( valRight == 0 )
				{
					attacks++;
				}
				else if( valUp == 0 )
				{
					attacks++;
				}
				else if( valDown == 0 )
				{
					attacks++;
				}
			}
			else if( map[spriteData.x-1][spriteData.y] == -3 && valLeft != -1 && 
					(valLeft <= valRight || valRight == -1) && 
					(valLeft <= valUp || valUp == -1) && 
					(valLeft <= valDown || valDown == -1))
			{
				lastMoveTime = curMoveTime;
				map[spriteData.x][spriteData.y] = -3; //free current spot
				spriteData.x--;
				map[spriteData.x][spriteData.y] = index;
				setDirec(DIREC_LEFT);
				Sprite::setFrames(3, 5);
			Sprite::setCurrentFrame(3);
			}
			else if( map[spriteData.x+1][spriteData.y] == -3 && valRight != -1 && 
					(valRight <= valUp || valUp == -1) &&
					(valRight <= valDown || valDown == -1))
			{
				lastMoveTime = curMoveTime;
				map[spriteData.x][spriteData.y] = -3; //free current spot
				spriteData.x++;
				map[spriteData.x][spriteData.y] = index;
				setDirec(DIREC_RIGHT);
				Sprite::setFrames(6, 8);
			Sprite::setCurrentFrame(6);
			}
			else if( map[spriteData.x][spriteData.y-1] == -3 && valUp != -1 && (valUp <= valDown || valDown == -1) )
			{
				lastMoveTime = curMoveTime;
				map[spriteData.x][spriteData.y] = -3; //free current spot
				spriteData.y--;
				map[spriteData.x][spriteData.y] = index;
				setDirec(DIREC_UP);
				Sprite::setFrames(9, 11);
			Sprite::setCurrentFrame(9);
			}
			else if( map[spriteData.x][spriteData.y+1] == -3 && valDown != -1)
			{
				lastMoveTime = curMoveTime;
				map[spriteData.x][spriteData.y] = -3; //free current spot
				spriteData.y++;
				map[spriteData.x][spriteData.y] = index;
				setDirec(DIREC_DOWN);
				Sprite::setFrames(0, 2);
			Sprite::setCurrentFrame(0);
			}
		}
		Sprite::update(frameTime);
	}
}

void Zombie::draw()
{
	if(alive)
	{
		Sprite::draw();
	}
}

void Zombie::onLostDevice()
{
	texture.onLostDevice();
}
void Zombie::onResetDevice()
{
	texture.onResetDevice();
}

int Zombie::getX()
{
	return spriteData.x;
}

int Zombie::getY()
{
	return spriteData.y;
}

void Zombie::setX(int x)
{
	spriteData.x = x;
}

void Zombie::setY(int y)
{
	spriteData.y = y;
}


#endif