// Programming 2D Games
// Copyright (c) 2011 by: 
// Charles Kelly
// Chapter 5 constants.h v1.0
#ifndef _CONSTANTS_H            // Prevent multiple definitions if this 
#define _CONSTANTS_H            // file is included in more than one place
#define WIN32_LEAN_AND_MEAN

#include <windows.h>

//-----------------------------------------------
// Useful macros
//-----------------------------------------------
// Safely delete pointer referenced item
#define SAFE_DELETE(ptr)       { if (ptr) { delete (ptr); (ptr)=NULL; } }
// Safely release pointer referenced item
#define SAFE_RELEASE(ptr)      { if(ptr) { (ptr)->Release(); (ptr)=NULL; } }
// Safely delete pointer referenced array
#define SAFE_DELETE_ARRAY(ptr) { if(ptr) { delete [](ptr); (ptr)=NULL; } }
// Safely call onLostDevice
#define SAFE_ON_LOST_DEVICE(ptr)    { if(ptr) { ptr->onLostDevice(); } }
// Safely call onResetDevice
#define SAFE_ON_RESET_DEVICE(ptr)   { if(ptr) { ptr->onResetDevice(); } }
#define TRANSCOLOR  SETCOLOR_ARGB(0,255,0,255)  // transparent color (magenta)

//-----------------------------------------------
//                  Constants
//-----------------------------------------------


// graphic images
const char BACKGROUND_IMAGE[]  = "..\\assets\\background.png"; 
const char RICK_IMAGE[]        = "..\\assets\\Rick.png";
const char ZOMBIE_IMAGE[]	   = "..\\assets\\Zombie.png";
const char HEART_IMAGE[]	   = "..\\assets\\Heart.png";

// game
const double PI = 3.14159265;
const float FRAME_RATE  = 200.0f;               // the target frame rate (frames/sec)
const float MIN_FRAME_RATE = 10.0f;             // the minimum frame rate
const float MIN_FRAME_TIME = 1.0f/FRAME_RATE;   // minimum desired time for 1 frame
const float MAX_FRAME_TIME = 1.0f/MIN_FRAME_RATE; // maximum time used in calculations
const int TILE_WIDTH = 50;
const int TILE_HEIGHT = 50;
const int GAME_TILE_WIDTH = 13;
const int GAME_TILE_HEIGHT = 10;
const short DIREC_UP = 1;
const short DIREC_DOWN = 2;
const short DIREC_LEFT = 3;
const short DIREC_RIGHT = 4;

// window
const char CLASS_NAME[] = "TWD";
const char GAME_TITLE[] = "The Walking Dead: Atlanta";
const bool FULLSCREEN = false;              // windowed or fullscreen
const UINT GAME_WIDTH =  TILE_WIDTH * GAME_TILE_WIDTH; // width of game in pixels
const UINT GAME_HEIGHT = TILE_HEIGHT * GAME_TILE_HEIGHT; // height of game in pixels
const int  RICK_START_FRAME = 0;            // starting frame of Rick animation
const int  RICK_END_FRAME = 3;              // last frame of Rick animation
const float RICK_ANIMATION_DELAY = 0.2f;    // time between frames of Rick animation
const int  RICK_COLS = 4;                   // Rick texture has 1 columns CHANGE WHEN SPRITESHEET IS MADE
const int ZOMBIE_COLS = 3;   //Zombie
const int  RICK_WIDTH = 50;                 // width of Rick image
const int  RICK_HEIGHT = 50;                // height of Rick image
const float ROTATION_RATE = 180.0f;         // degrees per second
const float SCALE_RATE = 0.2f;              // % change per second
//const float RICK_SPEED = 100.0f;            // pixels per second
const float RICK_SCALE = 1;              // starting Rick scale


// key mappings
// In this game simple constants are used for key mappings. If variables were used
// it would be possible to save and restore key mappings from a data file.
const UCHAR ESC_KEY			= VK_ESCAPE;       // escape key
const UCHAR ALT_KEY			= VK_MENU;         // Alt key
const UCHAR ENTER_KEY		= VK_RETURN;       // Enter key
const UCHAR LEFT_KEY		= VK_LEFT;     // left arrow
const UCHAR RIGHT_KEY		= VK_RIGHT;    // right arrow
const UCHAR UP_KEY			= VK_UP;       // up arrow
const UCHAR DOWN_KEY		= VK_DOWN;     // down arrow
const UCHAR SPACE_KEY		= VK_SPACE;    // space bar

#endif
