// Programming 2D Games
// Copyright (c) 2011 by: 
// Charles Kelly
// Chapter 5 spacewar.h v1.0

#ifndef _TWD_H             // Prevent multiple definitions if this 
#define _TWD_H             // file is included in more than one place
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <mmsystem.h>
#include "game.h"
#include "textureManager.h"
#include "sprite.h"
#include "Rick.h"
#include "Zombie.h"

//=============================================================================
// This class is the core of the game
//=============================================================================
class TWD : public Game
{
private:
    // game items
	int			   map[GAME_TILE_WIDTH][GAME_TILE_HEIGHT]; //A 2d array for the map.
														   //-3 is an empty tile that can be entered
														   //-2 is a generic full tile
														   //-1 is the tile containing Rick.
														   //0+ is a tile contiaining a zombie with that index in the
																//zombie array.
	int			         path[GAME_TILE_WIDTH][GAME_TILE_HEIGHT];
    TextureManager       backgroundTexture;   // background texture
	TextureManager       zombieTexture;		// Zombie texture
	TextureManager       heartTexture;
	Sprite               background;
	Sprite               heart;
	Rick                 rick;
	Zombie               z0;
	Zombie				 z1;
	Zombie			     z2;
	Zombie				 z3;
	Zombie				 z4;
	Zombie               z5;
	Zombie               z6;
	Zombie               z7;
	Zombie*              zombies;
	unsigned int         GameTime;
	unsigned int		 lastNewZombie;


public:
    // Constructor
    TWD();

    // Destructor
    virtual ~TWD();

    // Initialize the game
    void initialize(HWND hwnd);
    void update();      // must override pure virtual from Game
    void collisions();  // "
    void render();      // "
    void releaseAll();
    void resetAll();
};

TWD::TWD()
{

}

TWD::~TWD()
{
    releaseAll();           // call onLostDevice() for every graphics item
}

void TWD::initialize(HWND hwnd)
{
	zombies = new Zombie[16];
    Game::initialize(hwnd); // throws GameError
	for(int i=0; i<GAME_TILE_WIDTH; i++)
	{
		for(int j=0; j<GAME_TILE_HEIGHT; j++)
		{
			map[i][j] = -3;
			path[i][j] = -1;
		}
	}
    // background texture
    if (!backgroundTexture.initialize(graphics,BACKGROUND_IMAGE))
        throw(GameError(gameErrorNS::FATAL_ERROR, "Error initializing background texture"));
	// background image
    if (!background.initialize(graphics,0,0,0,&backgroundTexture))
        throw(GameError(gameErrorNS::FATAL_ERROR, "Error initializing background"));

	//heart texture and image
	if (!heartTexture.initialize(graphics,HEART_IMAGE))
        throw(GameError(gameErrorNS::FATAL_ERROR, "Error initializing heart texture"));
	if (!heart.initialize(graphics,0,0,0,&heartTexture))
        throw(GameError(gameErrorNS::FATAL_ERROR, "Error initializing heart"));
	heart.setTiled(false);
	PlaySound("..\\assets\\walkdead.wav",NULL,SND_FILENAME| SND_ASYNC |SND_LOOP|SND_NOSTOP);  //Plays music
	rick.initialize(map, path, graphics);
	z0.initialize(8,8,map,0,graphics);
	z1.initialize(9,9,map,1,graphics);
	z2.initialize(GAME_TILE_WIDTH,1,map,2,graphics);
	z3.initialize(GAME_TILE_WIDTH,2,map,3,graphics);
	z4.initialize(GAME_TILE_WIDTH,3,map,4,graphics);
	z5.initialize(GAME_TILE_WIDTH,4,map,5,graphics);
	z6.initialize(GAME_TILE_WIDTH,5,map,6,graphics);
	z7.initialize(GAME_TILE_WIDTH,6,map,7,graphics);
	zombies[0] = z0;
	zombies[1] = z1;
	zombies[2] = z2;
	zombies[3] = z3;
	zombies[4] = z4;
	zombies[5] = z5;
	zombies[6] = z6;
	zombies[7] = z7;
	for(int i=2; i<8; i++)
	{
		zombies[i].health = 0;
		zombies[i].alive = false;
	}

	lastNewZombie = GetTickCount();

    return;
}

void TWD::update()
{
	
    GameTime = GetTickCount();
	if( GameTime - lastNewZombie > 5000 ) //new zombie appears every 5 seconds
	{
		
		for(int i=0; i<8; i++ )
		{
			if( !zombies[i].alive )
			{
				zombies[i].setX(GAME_TILE_WIDTH);
				zombies[i].health = 1;
				zombies[i].alive = true;
				break;
			}
		}
		lastNewZombie = GameTime;
	}
	rick.update(map, path, zombies, input, frameTime);
	int attacks = 0;
	for(int i=0; i<8; i++)
	{
		zombies[i].update(map, i, path, input, frameTime, attacks);
	}
	rick.health -= attacks;
	for(int i=0; i<8; i++)
	{
		if(zombies[i].health <= 0 )
		{
			zombies[i].alive = false;
			map[zombies[i].getX()][zombies[i].getY()] = -3;
		}
	}
	if(rick.health <= 0 )
	{
		//GAME OVER
		//Should probably have some game over logic here.
	}
}


void TWD::collisions()
{
}

void TWD::render()
{
    graphics->spriteBegin();                // begin drawing sprites
	
    background.draw();                      // add the background to the scene
    rick.draw();

	for(int i=0; i<8; i++)
	{
		zombies[i].draw();
	}


	for(int i=0; i<rick.health; i++)
	{
		heart.setX(i*27);
		heart.draw();
	}
    graphics->spriteEnd();                  // end drawing sprites
}

void TWD::releaseAll()
{
    backgroundTexture.onLostDevice();
	rick.onLostDevice();

    Game::releaseAll();
    return;
}

void TWD::resetAll()
{
    backgroundTexture.onResetDevice();
	rick.onResetDevice();

    Game::resetAll();
    return;
}

#endif
